#ifndef PREPROCESS_H
#define PREPROCESS_H

#include "petscksp.h"
#include "petscmat.h"

#define TRANSFORMCOOKIE 2008
#define CHECKVALIDTRANSFORM(x) {if (!(x)) {printf("Null SalsaTransform"); return 1;} else if ((x)->cookie!=TRANSFORMCOOKIE) {printf("Invalid SalsaTransform"); return 1;}; }
/* ierr = SalsaTransformIntegrityTest(x); CHKERRQ(ierr); */
#define CHECKVALIDTRANSFORM0(x) {if (!(x)) {printf("Null SalsaTransform"); return 1;} else if ((x)->cookie!=TRANSFORMCOOKIE) {printf("Invalid SalsaTransform"); return 1;} }
#define TRANSFORMOBJECTCOOKIE 2009
#define CHECKVALIDTRANSFORMOBJECT(x) {if (!(x)) {printf("Null TransformObject"); return 1;} else if ((x)->cookie!=TRANSFORMOBJECTCOOKIE) {printf("Invalid TransformObject"); return 1;} }
#define TRUTH(x) ( (x) ? PETSC_TRUE : PETSC_FALSE )

//struct NumericalSolution {
//  void* dummy;
//} ;

typedef struct SystemPreprocessor_* SystemPreprocessor;
typedef struct NumericalProblem_* NumericalProblem;
typedef struct NumericalSolution_* NumericalSolution;
typedef PetscReal SuitabilityValue;

typedef struct SalsaTransform_* SalsaTransform;
typedef struct SalsaTransformObject_* SalsaTransformObject;

extern PetscErrorCode SysProInitialize();
extern PetscErrorCode SysProFinalize();
extern PetscErrorCode DeclarePreprocessor
(const char *name,
 PetscErrorCode(*global_setup)(),
 PetscErrorCode(*specific_setup)(NumericalProblem,SalsaTransform),
 PetscErrorCode(*specific_unset)(NumericalProblem),
 PetscErrorCode(*global_unset)(),
 PetscErrorCode(*ctxcreate)(NumericalProblem,void**),
 PetscErrorCode(*ctxdelete)(void*),
 PetscErrorCode(*start_function)(const char*,int,PetscTruth,NumericalProblem,NumericalProblem*,void*,void**,PetscTruth*),
 PetscErrorCode(*end_function) (const char*,PetscTruth,void*,void*,NumericalProblem,NumericalProblem,NumericalSolution,NumericalSolution)
 );
extern PetscErrorCode DeclarePreprocessorIntelligentChoice
(const char *name,PetscErrorCode(*picker)(NumericalProblem,const char**,const char**));
extern PetscErrorCode PreprocessorsOptionsHandling();

extern PetscErrorCode SysProDeclareFunctions
(PetscErrorCode(*classstaticsetup)(const char*),
 PetscErrorCode(*classdynamicsetup)(const char*,NumericalProblem),
 PetscErrorCode(*classproblemcloner)(const char*,const char*,int,NumericalProblem,NumericalProblem),
 PetscErrorCode(*solver)(NumericalProblem,void*,NumericalSolution*),
 PetscErrorCode(*problemdelete)(NumericalProblem),
 PetscErrorCode(*solutioncreator)(NumericalProblem,NumericalSolution*),
 PetscErrorCode(*solutioncopy)(NumericalSolution,NumericalSolution),
 PetscErrorCode(*solutiondelete)(NumericalSolution),
 PetscErrorCode(*ctxcloner)(const char*,const char*,void*,void**),
 PetscErrorCode(*ctxfree)(void*),
 PetscErrorCode(*solutioncontextdelete)(NumericalSolution)
 );

extern PetscErrorCode ProcessPreprocessorOptions(char *processor,void *ctx);

extern PetscErrorCode PreprocessorGetIndex(const char*,int*);
extern PetscErrorCode SystemPreprocessorGetByName(const char*,SystemPreprocessor*);
extern PetscErrorCode PreprocessorGetSetting(const char*,const char**,int*);

extern PetscErrorCode RetrievePreprocessorChoice(int,const char**,int*);
extern PetscErrorCode GetFirstPreprocessor(const char **preprocess);
extern PetscErrorCode GetNextPreprocessor(const char **next_one);

extern PetscErrorCode SuccessorPreprocessor(const char *,const char**);
extern PetscErrorCode InitRetrievingPreprocessors();
extern PetscErrorCode StartRetrievingCurrentPreprocessors(const char**,const char**,int*,PetscTruth*);
extern PetscErrorCode ContinueRetrievingCurrentPreprocessors(const char**,const char**,int*,PetscTruth*);
extern PetscErrorCode StartRetrievingAllPreprocessors(const char**,const char***,int*,PetscTruth*);
extern PetscErrorCode ContinueRetrievingAllPreprocessors(const char**,const char***,int*,PetscTruth*);
extern PetscErrorCode RetrieveAllPreprocessorValues(const char*,const char***,int*);
extern PetscErrorCode SysProShowOptions(MPI_Comm comm);

extern PetscErrorCode RegisterPreprocessorContext(const char*,void *ctx);
extern PetscErrorCode PreprocessorGetContext(const char*,void **ctx);

extern PetscErrorCode PreprocessedSolution
(const char*,NumericalProblem,void*,NumericalSolution*);
extern PetscErrorCode PreprocessedProblemSolving
(NumericalProblem,NumericalSolution*);

/*
 * Anamod interface
 */
extern PetscErrorCode PreprocessorSetPreservedCategories(const char*,const char*);
extern PetscErrorCode PreprocessorGetPreservedCategories(const char*,const char**);
extern PetscErrorCode SysProComputeQuantity
(NumericalProblem,const char*,const char*,void*,int*,PetscTruth*);
extern PetscErrorCode SysProRetrieveQuantity
(NumericalProblem,const char*,const char*,void*,int*,PetscTruth*);
extern PetscErrorCode SysProRemoveQuantity
(NumericalProblem,const char*,const char*,PetscTruth*);
extern PetscErrorCode SysProFreeQuantities(NumericalProblem);

/*
 * Tracing
 */
extern PetscErrorCode SysProDefaultTrace(void*,const char*,va_list);
extern PetscErrorCode SysProDeclareTraceFunction
(PetscErrorCode(*fn)(void*,const char*,va_list));
extern PetscErrorCode SysProDeclareTraceContext(void *ctx);
extern PetscErrorCode SysProTraceMessage(const char *fmt,...);
extern PetscErrorCode SysProHasTrace(PetscTruth *flg);
extern PetscErrorCode SysProDeclareProblemMonitor
(PetscErrorCode(*)(NumericalProblem));
extern PetscErrorCode SysProDeclareErrorTracer
(PetscErrorCode(*)(NumericalProblem,NumericalSolution,const char*));
extern PetscErrorCode SysProGetErrorTracer
(PetscErrorCode(**)(NumericalProblem,NumericalSolution,const char*));

/*
 * Output
 */
extern PetscErrorCode TabReportAllPreprocessors(const char **key,int);
extern PetscErrorCode TabReportActivePreprocessors(const char **key,const char **val,int);
extern PetscErrorCode ReportEnabledPreprocessors(const char*);
extern PetscErrorCode ReportSysProCallStackState(const char*);
extern PetscErrorCode ScreenOutputTab(const char*,const char*);
extern PetscErrorCode ScreenOutputTabLine(const char*,const char*);

/* Utility stuff */
extern PetscErrorCode NumericalProblemGetComm(NumericalProblem,MPI_Comm*);
extern PetscErrorCode TestSuitabilityContext(SalsaTransform,void*);

#endif
