#ifndef PREPROCESS_IMPL_H
#define PREPROCESS_IMPL_H

#include "petsc.h"
#include "syspro.h"

#define SYSPROCHECKVALID(i,c,s) {if (!i) SETERRQ1(1,"Null pointer for <%s>",s); if (i->cookie==0) SETERRQ1(1,"Not a valid <%s>; maybe object has already been freed",s); if (i->cookie!=c) SETERRQ1(1,"Not a valid <%s>",s);}
#define SYSPROCHECKVALIDa(i,c,s,a) {if (!i) SETERRQ2(1,"Null pointer for <%s>, argument %d",s,a); if (i->cookie==0) SETERRQ2(1,"Not a valid <%s>, argument %d; maybe object has already been freed",s,a); if (i->cookie!=c) SETERRQ2(1,"Not a valid <%s>, argument %d",s,a);}

#define NUMERICALPROBLEMHEADER \
  MPI_Comm comm; void *ctx;

struct NumericalProblem_ {
  NUMERICALPROBLEMHEADER
};

struct SystemPreprocessor_ {
  const char *name;
  SalsaTransform transform;
  const char *preserved,*required;
  PetscTruth exhaustive;
  PetscErrorCode(*setup)(NumericalProblem,SalsaTransform);
  PetscErrorCode(*unset)(NumericalProblem);
  PetscErrorCode(*ctxcreate)(NumericalProblem,void**);
  PetscErrorCode(*ctxdelete)(void*);
  PetscErrorCode(*start_function)(const char*,int,PetscTruth,NumericalProblem,NumericalProblem*,void*,void**,PetscTruth*);
  PetscErrorCode(*end_function)(const char*,PetscTruth,void*,void*,NumericalProblem,NumericalProblem,NumericalSolution,NumericalSolution);
  PetscErrorCode(*optionshandling)();
  PetscErrorCode(*intelligence)(NumericalProblem,const char**,const char**);
};

struct SalsaTransform_ {
  int cookie;
  char *name; PetscTruth userchoices;
  SalsaTransformObject *transformobjects;
  PetscErrorCode(*suitabilityctxtester)(void*);
  int alloc_objects,n_objects,*aprioriselection;
  int n_annotate_c; char **annotations_c;
  int n_annotate_i; char **annotations_i;
} ;

#endif
