#ifndef TRANSFORM_H
#define TRANSFORM_H

#include "syspro.h"
#include "petsc.h"

extern PetscErrorCode NewTransform(const char*,SalsaTransform*);
extern PetscErrorCode DeregisterTransform(SalsaTransform);
extern PetscErrorCode TransformGetName(SalsaTransform,const char**);
extern PetscErrorCode TransformGetByName(const char*,SalsaTransform*);

extern PetscErrorCode NewTransformObject(const char*,const char*,SalsaTransformObject*);
extern PetscErrorCode FreeTransformObject(SalsaTransformObject tf);
extern PetscErrorCode TransformObjectGetByName
(const char*,const char*,SalsaTransformObject*);
extern PetscErrorCode TransformGetObjects(SalsaTransform,int*,SalsaTransformObject**);
extern PetscErrorCode TransformObjectGetName(SalsaTransformObject,const char**);
extern PetscErrorCode TransformObjectGetTransformName(SalsaTransformObject,char**);
extern PetscErrorCode TransformObjectsGetNames(SalsaTransform,const char***);
extern PetscErrorCode TransformObjectSetExplanation(SalsaTransformObject,const char*);
extern PetscErrorCode TransformObjectSetSuitabilityFunction
(SalsaTransformObject,void*,PetscErrorCode(*)(NumericalProblem,void*,SuitabilityValue*));
extern PetscErrorCode SetSuitabilityContextTester(SalsaTransform,PetscErrorCode(*)(void*));
extern PetscErrorCode TransformObjectGetSuitabilityFunction
(SalsaTransformObject,void **,PetscErrorCode(**)(NumericalProblem,void*,SuitabilityValue*));
extern PetscErrorCode TransformObjectAddOptionExplanation(SalsaTransformObject,int,const char*);

extern PetscErrorCode TransformItemGetFirstOption(const char*,const char*,int*,PetscTruth*);
extern PetscErrorCode TransformItemGetNextOption(const char*,const char*,int*,PetscTruth*);

extern PetscErrorCode TransformReportTeXTable(SalsaTransform,FILE*);
extern PetscErrorCode TransformReportEnabled(SalsaTransform,const char**);
extern PetscErrorCode TransformItemDescribeShort(SalsaTransform,char*,int,char**);
extern PetscErrorCode TransformItemDescribeLong(SalsaTransform,char*,int,char**);

/*
 * Marking
 */
extern PetscErrorCode TransformObjectMark(SalsaTransformObject tf);
extern PetscErrorCode TransformObjectUnmark(SalsaTransformObject tf);
extern PetscErrorCode TransformObjectsMarkAll(SalsaTransform tf);
extern PetscErrorCode TransformObjectsUnmarkAll(SalsaTransform tf);
extern PetscErrorCode TransformObjectGetMark(SalsaTransformObject,int*);
extern PetscErrorCode TransformItemOptionMark(SalsaTransform,const char*,int);
extern PetscErrorCode TransformGetNUnmarked(SalsaTransform,int*);
extern PetscErrorCode TransformObjectsUseOnly(SalsaTransform,const char*);
extern PetscErrorCode TransformItemOptionsUseOnly(SalsaTransformObject,const char*);
extern PetscErrorCode TransformGetNItems(SalsaTransform,int *n);
extern PetscErrorCode TransformGetNextUnmarkedItem
(SalsaTransform,const char*,SalsaTransformObject*,PetscTruth*);
extern PetscErrorCode PreprocessorSaveAprioriSelection(SystemPreprocessor);
extern PetscErrorCode PreprocessorApplyAprioriSelection(SystemPreprocessor);

/*
 * annotations
 */
extern PetscErrorCode SysProDefineCharAnnotation(const char*,const char*);
extern PetscErrorCode TransformCharAnnotationGetIndex(SalsaTransform,const char*,int*,PetscTruth*);
extern PetscErrorCode TransformObjectCharAnnotate(SalsaTransformObject,const char*,const char*);
extern PetscErrorCode TransformObjectIntAnnotate(SalsaTransformObject,const char*,int);
extern PetscErrorCode TransformObjectGetIntAnnotation
(SalsaTransformObject,const char*,int*,PetscTruth*);
extern PetscErrorCode SysProDefineIntAnnotation(const char*,const char*);
extern PetscErrorCode TransformIntAnnotationGetIndex
(SalsaTransform,const char*,int*,PetscTruth*);
extern PetscErrorCode TransformItemIntAnnotate(SalsaTransform,int,int);
extern PetscErrorCode TransformItemGetIntAnnotation
(SalsaTransform,int idx,char *an,int *v,PetscTruth *f);

/*
 * options
 */
extern PetscErrorCode TransformObjectDefineOption(SalsaTransformObject,const char*);
extern PetscErrorCode TransformObjectAddOption(SalsaTransformObject,int);

extern PetscErrorCode TransformItemCharAnnotationGetIndex(SalsaTransform,char*,int*);
extern PetscErrorCode TransformItemGetCharAnnotation
(SalsaTransform,int idx,char *an,char **v,PetscTruth*);


extern PetscErrorCode TransformCurrentItemDefineOption(SalsaTransform,char*,char*);
extern PetscErrorCode TransformItemDefineOption(SalsaTransform,int,char*,char*);
extern PetscErrorCode TransformItemGetNOptions(SalsaTransform,int it,int *nopt);
extern PetscErrorCode TransformItemGetOptionI(SalsaTransform,int it,int iopt,int *v);

extern PetscErrorCode TransformSetUserChoices(SalsaTransform,PetscTruth);
extern PetscErrorCode TransformGetUserChoices(SalsaTransform,PetscTruth*);
extern PetscErrorCode SalsaTransformIntegrityTest(SalsaTransform tf);

#endif
